/*******************************************************************************
 * Copyright 2015-2016 Radosław Piliszek
 *
 * This file is part of WSX2MIZ.
 *
 * WSX2MIZ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WSX2MIZ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with WSX2MIZ.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package net.piliszek.mizar.wsx2miz;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.HashSet;
import java.util.Set;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

public class DCX {

	private static Set<String> builtins = new HashSet<>();

	static {
		builtins.add("(#");
		builtins.add("#)");
	}

	private static XMLInputFactory xif = XMLInputFactory.newInstance();

	private static Set<String> operators;
	private static Set<String> leftBrackets;
	private static Set<String> rightBrackets;

	private static QName kindQName = new QName("kind");
	private static QName nameQName = new QName("name");

	public static void load(File f)
			throws FileNotFoundException, XMLStreamException {
		operators = new HashSet<>();
		leftBrackets = new HashSet<>();
		rightBrackets = new HashSet<>();

		XMLEventReader xer = xif.createXMLEventReader(
				new BufferedInputStream(new FileInputStream(f)));

		while (xer.hasNext()) {
			XMLEvent xe = xer.nextEvent();
			if (xe.isStartElement()) {
				StartElement se = xe.asStartElement();
				String elemName = se.getName().getLocalPart();

				if (elemName == "Symbol") {
					String kind = se.getAttributeByName(kindQName).getValue();
					String name = se.getAttributeByName(nameQName).getValue();
					if (kind.equals("O")) { // operation functor (operator)
						operators.add(name);
					} else if (kind.equals("K")) { // left bracket
						leftBrackets.add(name);
					} else if (kind.equals("L")) { // right bracket
						rightBrackets.add(name);
					}
				}
			}
		}
	}

	public static boolean isOperator(String s) {
		return operators.contains(s);
	}

	public static boolean isLeftBracket(String s) {
		return leftBrackets.contains(s);
	}

	public static boolean isRightBracket(String s) {
		return rightBrackets.contains(s);
	}

	public static boolean isOperatorLike(String s) {
		return builtins.contains(s)
				|| isOperator(s)
				|| isLeftBracket(s)
				|| isRightBracket(s);
	}

	public static boolean isAnyPrefixOperatorLike(String p, String s) {
		for (int i = 1; i <= s.length(); i++) {
			if (isOperatorLike(p + s.substring(0, i))) {
				return true;
			}
		}
		return false;
	}

	public static boolean isAnySuffixOperatorLike(String p, String s) {
		for (int i = s.length() - 1; i >= 0; i--) {
			if (isOperatorLike(s.substring(i) + p)) {
				return true;
			}
		}
		return false;
	}

}
