/*******************************************************************************
 * Copyright 2015-2016 Radosław Piliszek
 *
 * This file is part of WSX2MIZ.
 *
 * WSX2MIZ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WSX2MIZ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with WSX2MIZ.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package net.piliszek.mizar.wsx2miz.schema;

import java.io.IOException;
import java.util.List;

import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlSeeAlso;

import net.piliszek.mizar.wsx2miz.MizWriter;

@XmlSeeAlso({
	ConjunctiveFormula.class,
	DisjunctiveFormula.class,
	FlexaryConjunctiveFormula.class,
	FlexaryDisjunctiveFormula.class,
	ConditionalFormula.class,
	BiconditionalFormula.class
})
public abstract class BinaryFormula extends Formula {

	@XmlElementRef(required = true)
	protected List<Formula> formulas;

	protected Formula formulaL;
	protected Formula formulaR;

	protected void afterUnmarshal(Unmarshaller unmarshaller, Object parent) {
		formulaL = formulas.get(0);
		formulaR = formulas.get(1);
	}

	protected abstract String getSpelling();

	protected abstract boolean isAssociative();

	protected boolean needsParentheses(Formula f) {
		try {
			BinaryFormula bf = (BinaryFormula) f;
			return isAssociative() && bf.getClass() != getClass() || !bf.isAssociative();
		} catch (ClassCastException e) {
			return false;
		}
	}

	@Override
	public void write(MizWriter w, int flags) throws IOException {
		if (needsParentheses(formulaL)) {
			w.write("(");
			formulaL.write(w, UNSAFE_LEFT_PAREN | UNSAFE_RIGHT_PAREN);
			w.write(")");
		} else {
			formulaL.write(w, UNSAFE_RIGHT_DANGLING_FORMULA | (UNSAFE_LEFT_MASK & flags));
		}

		w.ensureSpace();
		w.write(getSpelling());
		w.ensureSpace();

		w.beginSubExp();
		if (needsParentheses(formulaR)) {
			w.write("(");
			formulaR.write(w, UNSAFE_LEFT_PAREN | UNSAFE_RIGHT_PAREN);
			w.write(")");
		} else {
			formulaR.write(w, UNSAFE_RIGHT_MASK & flags);
		}
		w.endSubExp();
	}

}
