/*******************************************************************************
 * Copyright 2015-2016 Radosław Piliszek
 *
 * This file is part of WSX2MIZ.
 *
 * WSX2MIZ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WSX2MIZ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with WSX2MIZ.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package net.piliszek.mizar.wsx2miz.schema;

import java.io.IOException;
import java.util.List;

import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

import net.piliszek.mizar.wsx2miz.MizWriter;

@XmlType(propOrder = { "schemeHead", "reasoningItems", "perCases",
	"caseBlocks" })
@XmlRootElement(name = "Block-Scheme-Block")
public class BlockSchemeBlock extends Block {

	@XmlElementRef(required = true)
	protected ItemSchemeHead schemeHead;

	@XmlElementRef(required = false)
	protected List<ReasoningItem> reasoningItems;

	@XmlElementRef(required = false)
	protected ItemPerCases perCases;

	@XmlElementRef(required = false)
	protected List<ItemCaseBlock> caseBlocks;

	@Override
	public void write(MizWriter w) throws IOException {
		w.writeIndent();
		w.write("scheme");
		w.newLine();
		w.increaseIndent();
		w.writeIndent();
		w.write(schemeHead.scheme);
		w.write(schemeHead.schematicVariables);
		w.write(":");
		w.ensureSpace();
		w.write(schemeHead.formula);
		w.newLine();
		w.decreaseIndent();
		if (null != schemeHead.provisionalFormulas) {
			w.writeIndent();
			w.write("provided");
			w.newLine();
			w.increaseIndent();
			w.write(schemeHead.provisionalFormulas);
			w.newLine();
			w.decreaseIndent();
		}
		w.writeIndent();
		w.write("proof");
		w.newLine();
		w.writeReasoning(reasoningItems, perCases, caseBlocks);
		w.writeIndent();
		w.write("end");
	}

}
