/*******************************************************************************
 * Copyright 2015-2016 Radosław Piliszek
 *
 * This file is part of WSX2MIZ.
 *
 * WSX2MIZ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WSX2MIZ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with WSX2MIZ.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package net.piliszek.mizar.wsx2miz.schema;

import java.io.IOException;
import java.util.List;

import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlRootElement;

import net.piliszek.mizar.wsx2miz.MizWriter;

@XmlRootElement(name = "Fraenkel-Term")
public class FraenkelTerm extends Term {

	@XmlElementRef(required = false)
	protected List<ImplicitlyQualifiedSegment> implicitlyQualifiedSegments;
	@XmlElementRef(required = false)
	protected List<ExplicitlyQualifiedSegment> explicitlyQualifiedSegments;
	@XmlElementRef(required = true)
	protected Term term;
	@XmlElementRef(required = true)
	protected Formula formula;

	@Override
	public void write(MizWriter w) throws IOException {
		w.write("{");
		term.write(w);
		if (null != implicitlyQualifiedSegments) {
			w.ensureSpace();
			w.write("where");
			w.ensureSpace();
			w.writeImplicitlyQualifiedSegments(implicitlyQualifiedSegments);
		}
		if (null != explicitlyQualifiedSegments) {
			w.ensureSpace();
			w.write("where");
			w.ensureSpace();
			w.writeExplicitlyQualifiedSegments(explicitlyQualifiedSegments,
					"is", 0);
		}
		w.ensureSpace();
		w.write(":");
		w.ensureSpace();
		formula.write(w);
		w.write("}");
	}

}
