/*******************************************************************************
 * Copyright 2015-2016 Radosław Piliszek
 *
 * This file is part of WSX2MIZ.
 *
 * WSX2MIZ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WSX2MIZ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with WSX2MIZ.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package net.piliszek.mizar.wsx2miz.schema;

import java.io.IOException;
import java.util.List;

import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

import net.piliszek.mizar.wsx2miz.MizWriter;

@XmlType(propOrder = { "proposition", "justification", "iterativeSteps" })
@XmlRootElement(name = "Item-Conclusion-Iterative-Equality")
public class ItemConclusionIterativeEquality extends ItemConclusion {

	@XmlElementRef(required = true)
	protected Proposition proposition;

	@XmlElementRef(required = true)
	protected SimpleJustification justification;

	@XmlElementRef(required = true)
	protected List<IterativeStep> iterativeSteps;

	public boolean isLinked() {
		return justification.hasLink();
	}

	@Override
	public void write(MizWriter w) throws IOException {
		w.writeIndent();
		if (isLinked()) {
			w.write("hence");
		} else {
			w.write("thus");
		}
		w.ensureSpace();

		if (proposition.label.exists()) {
			proposition.label.write(w);
			w.ensureSpace();
		}

		proposition.formula.write(w);

		w.writeJustification(justification);

		for (IterativeStep step : iterativeSteps) {
			w.newLine();
			w.writeIndent();
			w.write(".=");
			w.ensureSpace();
			step.write(w);
		}

		w.write(";");
		w.newLine();
	}

}
