/*******************************************************************************
 * Copyright 2015-2016 Radosław Piliszek
 *
 * This file is part of WSX2MIZ.
 *
 * WSX2MIZ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WSX2MIZ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with WSX2MIZ.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package net.piliszek.mizar.wsx2miz.schema;

import java.io.IOException;
import java.util.LinkedList;

import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

import net.piliszek.mizar.wsx2miz.MizWriter;

@XmlType(propOrder = { "variable", "term" })
@XmlRootElement(name = "Item-Exemplification")
public class ItemExemplification extends SkeletonItem {

	@XmlElementRef(required = false)
	protected Variable variable;

	@XmlElementRef(required = false)
	protected Term term;

	protected LinkedList<Example> examples;

	public void addExample(Example e) {
		examples.add(e);
	}

	public Example getFirstExample() {
		return examples.getFirst();
	}

	protected void afterUnmarshal(Unmarshaller unmarshaller, Object parent) {
		examples = new LinkedList<>();
		if (variable != null && term != null) {
			addExample(new Equality(variable, term));
		} else if (variable != null) {
			addExample(variable);
		} else if (term != null) {
			addExample(term);
		} else {
			throw new RuntimeException("BAD ItemExemplification - no child present");
		}
	}

	@Override
	public void write(MizWriter w) throws IOException {
		w.writeIndent();
		w.write("take");
		w.ensureSpace();
		w.writeExamples(examples);
		w.write(";");
		w.newLine();
	}

}
