/*******************************************************************************
 * Copyright 2015-2016 Radosław Piliszek
 *
 * This file is part of WSX2MIZ.
 *
 * WSX2MIZ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WSX2MIZ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with WSX2MIZ.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package net.piliszek.mizar.wsx2miz.schema;

import java.io.IOException;

import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlSeeAlso;
import javax.xml.bind.annotation.XmlType;

import net.piliszek.mizar.wsx2miz.MizWriter;

@XmlSeeAlso({
	ItemFunctorDefinitionEquals.class,
	ItemFunctorDefinitionMeans.class
})
@XmlType(propOrder = { "redefine", "functorPattern", "typeSpecification",
	"definiens" })
public abstract class ItemFunctorDefinitionWithDefiniens
		extends ItemFunctorDefinition {

	@XmlElementRef(required = false)
	protected Redefine redefine;

	@XmlElementRef(required = true)
	protected FunctorPattern functorPattern;

	@XmlElementRef(required = false)
	protected TypeSpecification typeSpecification;

	@XmlElementRef(required = true)
	protected Definiens definiens;

	protected abstract String getSpelling();

	protected void afterUnmarshal(Unmarshaller unmarshaller, Object parent) {
		functorPattern.register();
	}

	@Override
	public void write(MizWriter w) throws IOException {
		w.writeIndent();

		if (redefine != null) {
			w.write("redefine");
			w.ensureSpace();
		}

		w.write("func");
		w.ensureSpace();

		functorPattern.write(w);

		if (typeSpecification != null) {
			w.ensureSpace();
			w.write(typeSpecification);
		}

		w.ensureSpace();
		w.write(getSpelling());
		w.ensureSpace();

		definiens.write(w);

		w.write(";");
		w.newLine();
	}

}
