/*******************************************************************************
 * Copyright 2015-2016 Radosław Piliszek
 *
 * This file is part of WSX2MIZ.
 *
 * WSX2MIZ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WSX2MIZ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with WSX2MIZ.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package net.piliszek.mizar.wsx2miz.schema;

import java.io.IOException;
import java.util.List;

import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

import net.piliszek.mizar.wsx2miz.MizWriter;

@XmlType(propOrder = { "variablesOrEqualities", "type", "justification" })
@XmlRootElement(name = "Item-Type-Changing-Statement")
public class ItemTypeChangingStatement extends ItemStatement {

	@XmlElementRef(required = true)
	protected List<VariableOrEquality> variablesOrEqualities;

	@XmlElementRef(required = true)
	protected Type type;

	@XmlElementRef(required = true)
	protected Justification justification;

	public boolean isLinked() {
		return justification.hasLink();
	}

	@Override
	public void write(MizWriter w) throws IOException {
		w.writeIndent();
		if (isLinked()) {
			w.write("then");
			w.ensureSpace();
		}

		w.write("reconsider");
		w.ensureSpace();
		w.writeExamples(variablesOrEqualities);
		w.ensureSpace();

		w.beginSubExp();
		w.write("as");
		w.ensureSpace();
		type.write(w);
		w.endSubExp();

		w.writeJustification(justification);

		w.write(";");
		w.newLine();
	}

}
