/*******************************************************************************
 * Copyright 2015-2016 Radosław Piliszek
 *
 * This file is part of WSX2MIZ.
 *
 * WSX2MIZ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WSX2MIZ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with WSX2MIZ.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package net.piliszek.mizar.wsx2miz.schema;

import java.io.IOException;
import java.util.List;

import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlRootElement;

import net.piliszek.mizar.wsx2miz.FRM;
import net.piliszek.mizar.wsx2miz.MizWriter;

@XmlRootElement(name = "Operation-Functor-Pattern")
public class OperationFunctorPattern extends FunctorPattern {

	@XmlAttribute(name = "nr", required = true)
	protected int nr;
	@XmlAttribute(name = "spelling", required = true)
	protected String spelling;

	@XmlElementRef(required = true)
	protected List<Loci> loci;

	private Loci lociL;
	private Loci lociR;

	protected void afterUnmarshal(Unmarshaller unmarshaller, Object parent) {
		lociL = loci.get(0);
		lociR = loci.get(1);
	}

	@Override
	public void register() {
		FRM.registerFunctorFormat(nr, lociL.count(), lociR.count());
	}

	@Override
	public void write(MizWriter w) throws IOException {
		if (lociL.arePresent()) {
			boolean manyLociL = lociL.areMoreThanOne();
			if (manyLociL) {
				w.write("(");
			}
			w.write(lociL);
			if (manyLociL) {
				w.write(")");
			}
			w.ensureSpace();
		}
		w.write(spelling);
		if (lociR.arePresent()) {
			w.ensureSpace();
			boolean manyLociR = lociR.areMoreThanOne();
			if (manyLociR) {
				w.write("(");
			}
			w.write(lociR);
			if (manyLociR) {
				w.write(")");
			}
		}
	}

}
