/*******************************************************************************
 * Copyright 2015-2016 Radosław Piliszek
 *
 * This file is part of WSX2MIZ.
 *
 * WSX2MIZ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WSX2MIZ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with WSX2MIZ.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package net.piliszek.mizar.wsx2miz.schema;

import java.io.IOException;
import java.util.List;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlType;

import net.piliszek.mizar.wsx2miz.MizWriter;

@XmlType(propOrder = { "references" })
@XmlRootElement(name = "Scheme-Justification")
public class SchemeJustification extends SimpleJustification {

	@XmlAttribute(name = "idnr", required = true)
	protected int idnr;

	@XmlAttribute(name = "nr", required = true)
	protected int nr;

	@XmlAttribute(name = "poscol", required = true)
	protected int poscol;

	@XmlAttribute(name = "posline", required = true)
	protected int posline;

	@XmlAttribute(name = "spelling", required = true)
	protected String spelling;

	@XmlElementRef(required = false)
	protected List<Reference> references;

	@Override
	public boolean hasLink() {
		return false;
	}

	@Override
	public boolean hasReferences() {
		// always true because it always refers to a scheme (local or not)
		return true;
	}

	@Override
	public void write(MizWriter w) throws IOException {
		w.beginSubExp();
		if (nr == 0) { // local scheme
			w.write(String.format("from %s", spelling));
		} else {
			w.write(String.format("from %s:sch %d", spelling, idnr));
		}
		if (references != null) {
			w.write("(");
			String d = "";
			for (Reference r : references) {
				w.write(d);
				r.write(w);
				d = ",";
			}
			w.write(")");
		}
		w.endSubExp();
	}

}
