/*******************************************************************************
 * Copyright 2015-2016 Radosław Piliszek
 *
 * This file is part of WSX2MIZ.
 *
 * WSX2MIZ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WSX2MIZ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with WSX2MIZ.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package net.piliszek.mizar.wsx2miz.schema;

import java.io.IOException;
import java.util.List;

import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlRootElement;

import net.piliszek.mizar.wsx2miz.MizWriter;

@XmlRootElement(name = "Simple-Fraenkel-Term")
public class SimpleFraenkelTerm extends Term {

	@XmlElementRef(required = false)
	protected List<ExplicitlyQualifiedSegment> qualifiedSegments;

	@XmlElementRef(required = true)
	protected Term term;

	private void writeTerm(MizWriter w) throws IOException {
		w.write("the set of all");
		w.ensureSpace();
		w.write(term);
	}

	private void writeFull(MizWriter w, int flags) throws IOException {
		writeTerm(w);
		w.ensureSpace();
		w.write("where");
		w.ensureSpace();
		w.writeExplicitlyQualifiedSegments(qualifiedSegments, "is", flags);
	}

	@Override
	public void write(MizWriter w, int flags) throws IOException {
		if (null != qualifiedSegments) {
			if ((flags & UNSAFE_RIGHT_DANGLING_COMMA) != 0) {
				w.write("(");
				writeFull(w, 0);
				w.write(")");
			} else {
				writeFull(w, UNSAFE_RIGHT_MASK & flags);
			}
		} else if ((flags & UNSAFE_RIGHT_DANGLING_TERM) != 0) {
			w.write("(");
			writeTerm(w);
			w.write(")");
		} else {
			writeTerm(w);
		}
	}

}
