/*******************************************************************************
 * Copyright 2015-2016 Radosław Piliszek
 *
 * This file is part of WSX2MIZ.
 *
 * WSX2MIZ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WSX2MIZ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with WSX2MIZ.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package net.piliszek.mizar.wsx2miz.schema;

import java.io.IOException;
import java.util.List;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlRootElement;

import net.piliszek.mizar.wsx2miz.FRM;
import net.piliszek.mizar.wsx2miz.MizWriter;

@XmlRootElement(name = "Standard-Type")
public class StandardType extends Type {

	@XmlAttribute(name = "nr", required = true)
	protected int nr;
	@XmlAttribute(name = "spelling", required = true)
	protected String spelling;

	@XmlElementRef(required = false)
	protected List<Term> terms;

	@Override
	public boolean isAmbiguous() {
		return FRM.isModeAmbiguous(nr);
	}

	@Override
	public void write(MizWriter w, int flags) throws IOException {
		if (null != terms) {
			if ((flags & UNSAFE_RIGHT_DANGLING_TERM) != 0
					|| ((flags & UNSAFE_RIGHT_DANGLING_COMMA) != 0
							&& isAmbiguous())) {
				w.write("(");
			}
			w.write(spelling);
			w.ensureSpace();
			w.write("of");
			w.ensureSpace();
			String d = "";
			for (Term t : terms) {
				w.write(d);
				t.write(w);
				d = ",";
			}
			if ((flags & UNSAFE_RIGHT_DANGLING_TERM) != 0
					|| ((flags & UNSAFE_RIGHT_DANGLING_COMMA) != 0
							&& isAmbiguous())) {
				w.write(")");
			}
		} else {
			w.write(spelling);
		}
	}

	@Override
	public void write(MizWriter w) throws IOException {
		write(w, 0);
	}

}
