/*******************************************************************************
 * Copyright 2015-2016 Radosław Piliszek
 *
 * This file is part of WSX2MIZ.
 *
 * WSX2MIZ is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * WSX2MIZ is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with WSX2MIZ.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package net.piliszek.mizar.wsx2miz.schema;

import java.io.IOException;

import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlRootElement;

import net.piliszek.mizar.wsx2miz.MizWriter;

@XmlRootElement(name = "Universal-Quantifier-Formula")
public class UniversalQuantifierFormula extends QuantifiedFormula {

	protected Formula antecedent;

	private void optimize() {
		try {
			ConditionalFormula cf = (ConditionalFormula) formula;
			antecedent = cf.getAntecedent();
			formula = cf.getConsequent();
		} catch (ClassCastException e) {
			// ignore it
		}
	}

	protected void afterUnmarshal(Unmarshaller unmarshaller, Object parent) {
		optimize();
	}

	@Override
	public void write(MizWriter w, int flags) throws IOException {
		if ((flags & UNSAFE_RIGHT_DANGLING_FORMULA) != 0) {
			w.write("(");
		}
		w.write("for");
		w.ensureSpace();
		qualifiedSegment.write(w, "being", 0);
		w.ensureSpace();
		if (null != antecedent) {
			w.write("st");
			w.ensureSpace();
			w.beginSubExp();
			antecedent.write(w);
			w.endSubExp();
			w.ensureSpace();
		}
		if (!(formula instanceof QuantifiedFormula)) {
			w.write("holds");
			w.ensureSpace();
		}
		w.beginSubExp();
		formula.write(w);
		w.endSubExp();
		if ((flags & UNSAFE_RIGHT_DANGLING_FORMULA) != 0) {
			w.write(")");
		}
	}

}
